;una lista associativa e' una lista di binding. Ogni binding associa un
;valore numerico ad un nome (simbolo)
;
;funzione di ricerca in una lista associativa, dato un simbolo ed una
;lista associativa restituisce il valore associato al simbolo nella
;lista associativa. Ritorna un errore nel caso il nome non sia legato
;nella lista
(define-struct binding (name value))
(define (lookup name l)
  (cond
    [(empty? l)
     (error 'lookup
            (string-append
             (symbol->string name)
             " "
             "not found"))]
    [(symbol=? (binding-name (first l)) name)
     (binding-value (first l))]
    [else (lookup name (rest l))]))

(define lista1 (list (make-binding 'a 1)
                     (make-binding 'b 2)))

; esercizio: dati in input una chiave ed una lista associativa,
; restituire in output tutti i valori (ovvero una lista di valori)
; associati alla chiave nella lista associativa data e' necessario il
; messaggio di errore not found?

; un albero di espressioni e':
; - un numero
; - un nodo somma avente due figli alberi
; - un nodo prodotto avente due figli alberi

(define-struct piu (arg1 arg2))
(define-struct per (arg1 arg2))

; rappresentazione della espressione scheme
; (+ 1 (* 2 3))
(define expr1 (make-piu 1 (make-per 2 3)))

;funzione di valutazione eval1: espressione -> numero
(define (eval1 expr)
  (cond
    [(number? expr) expr]
    [(piu? expr)
     (+ (eval1 (piu-arg1 expr))
        (eval1 (piu-arg2 expr)))]
    [(per? expr)
     (* (eval1 (per-arg1 expr))
        (eval1 (per-arg2 expr)))]
    [else (error 'eval1 "not an expression")]))

;proviamo la funzione eval1:
;(eval1 expr1)

;aumentiamo la nostra definizione di alberi di  espression aggiungendo
;come foglie dell'albero i simboli. Ogni simbolo rappresenta
;l'occorrenza di una variabile in un'espressione scheme
;esempio, rappresentazione dell'espressione scheme:
;(+ 1 (* a b))
(define expr2 (make-piu 1 (make-per 'a 'b)))

;funzione di valutazione di alberi di espressioni che possono contenere
;variabili
(define (eval2 env expr)
  (cond
    [(number? expr) expr]
    [(symbol? expr) (eval2 env (lookup expr env))]
    [(piu? expr)
     (+ (eval2 env (piu-arg1 expr))
        (eval2 env (piu-arg2 expr)))]
    [(per? expr)
     (* (eval2 env (per-arg1 expr))
        (eval2 env (per-arg2 expr)))]
    [else (error 'eval2 "not an expression")]))

(define env2 (list (make-binding 'a 1)
                   (make-binding 'b 2)))

;proviamo la funzione eval2:
;(eval2 env2 expr2)

; aumentiamo la definizione di alberi di espressioni in modo che possa
; contenere nodi applicazione. Un nodo di questo tipo ha come figlio sx
; un simbolo (il nome della funzione) e come figlio dx una espressione
; (l'argomento della funzione)
(define-struct appl (name arg))

;esempio, rappresentazione dell'espressione scheme:
;(double (+ 1 2))
(define expr3 (make-appl 'double
                         (make-piu 1 2)))

;la struttura fun rappresenta una definizione scheme di funzione ad un
;solo argomento
;<name> rappresenta il nome della funzione ed e' un simbolo
;<arg> rappresenta il nome dell'argomento della funzione, anch'esso un
;simbolo
;<body> rappresenta il corpo della funzione: e' una espressione e puo'
;contenere occorrenze della variabile corrispondente all'argomento della
;funzione
(define-struct fun (name arg body))

;esempio, rappresentazione della definizione scheme:
;(define (double x) (+ x x))
(define fun3 (make-fun 'double
                       'x
                       (make-piu 'x 'x)))

;funzione di valutazione di alberi di espressioni che possono contenere
;applicazioni di funzioni
(define (eval3 env expr)
  (cond
    [(number? expr) expr]
    [(symbol? expr) (eval3 env (lookup expr env))]
    [(piu? expr)
     (+ (eval3 env (piu-arg1 expr))
        (eval3 env (piu-arg2 expr)))]
    [(per? expr)
     (* (eval3 env (per-arg1 expr))
        (eval3 env (per-arg2 expr)))]
    [(appl? expr)
     (eval3
      (cons
       (make-binding
        (fun-arg (lookup (appl-name expr) env))
        (eval3 env (appl-arg expr)))
       env)
      (fun-body (lookup (appl-name expr) env)))]
    [else (error 'eval3 "not an expression")]))

(define env3 (list (make-binding 'double fun3)))

;proviamo la funzione eval3;
;(eval3 env3 expr3)
