; esercizio subst
(define-struct op (name arg1 arg2))
(define t (make-op 'piu
                   (make-op 'per 1 2)
                   'x))
(define-struct binding (name value))
(define (lookup name l)
  (cond
    [(empty? l)
     (error 'lookup
            (string-append
             (symbol->string name)
             " "
             "not found"))]
    [(symbol=? (binding-name (first l)) name)
     (binding-value (first l))]
    [else (lookup name (rest l))]))
(define env (list (make-binding 'x 10)))

(define (subst t env)
  (cond
    [(number? t) t]
    [(symbol? t) (lookup t env)]
    [(op? t) (make-op (op-name t)
                      (subst (op-arg1 t) env)
                      (subst (op-arg2 t) env))]))

;esercizi su alberi generalizzati

;definizione, un albero generalizzato e' una coppia
;<valore, figli> dove valore e' un numero e figli
;e' una lista (eventualmente vuota) di alberi
(define-struct tree (value children))
(define gt
  (make-tree 1 (list (make-tree 2
                                (list (make-tree 7 empty)))
                     (make-tree 5 empty)
                     (make-tree 4
                                (list (make-tree 5 empty)
                                      (make-tree 6 empty))))))

;funzioni su alberi generalizzati (di numeri)
;tassonomia
;- funzioni che trattano uniformemente i nodi
;  - funzioni il cui valore di ritorno non dipende dal
;    valore del nodo

;funzione card: albero -> numero di nodi contenuti nell'albero
(define (card t)
  (+ 1 (card-children (tree-children t))))
;funzione card-children: lista di figli -> numero di nodi
;contenuti in tutti i figli
(define (card-children l)
  (cond
    [(empty? l) 0]
    [else (+ (card (first l)) (card-children (rest l)))]))

;- funzioni che trattano uniformemente i nodi
;  - funzioni il cui valore di ritorno dipende dal valore
;    del nodo

;visit: albero -> lista dei numeri contenuti nell'albero
(define (visit t)
  (cons (tree-value t)
        (visit-children (tree-children t))))
;visit children: lista di alberi -> lista dei numeri contenuti
;in ogni albero appartenente alla lista di input
(define (visit-children l)
  (cond
    [(empty? l) empty]
    [else
     (append (visit (first l))
             (visit-children (rest l)))]))
;esercizio: implementare la postvisita (i.e. visita prima i
;sottoalberi e poi il nodo corrente)

;sum: albero -> somma di tutti i numeri contenuti nell'albero
(define (sum t)
  (+ (tree-value t)
     (sum-children (tree-children t))))
;sum-children: lista di alberi -> somma di tutti i numeri
;contentui in tutti gli alberi della lista
(define (sum-children l)
  (cond
    [(empty? l) 0]
    [else (+ (sum (first l))
             (sum-children (rest l)))]))

;- funzioni che non trattano uniformemente i nodi
;  - visita completa dell'albero

;count: valore + albero -> numero di occorrenze di quel valore
;nell'albero
(define (count value t)
  (cond
    [(= (tree-value t) value)
     (+ 1 (count-children value (tree-children t)))]
    [else (count-children value (tree-children t))]))
;count-children: valore + lista di alberi -> numero di
;occorrenze del valore in ogni albero contenuto nella lista
(define (count-children value l)
  (cond
    [(empty? l) 0]
    [else
     (+ (count value (first l))
        (count-children value (rest l)))]))

(define (leaves t)
  (cond
    [(empty? (tree-children t)) (list (tree-value t))]
    [else (leaves-children (tree-children t))]))
(define (leaves-children l)
  (cond
    [(empty? l) empty]
    [else (append (leaves (first l))
                  (leaves-children (rest l)))]))

;- funzioni che non trattano uniformemente i nodi
;  - visita parziale dell'albero

;come definire odd
(define (dispari? n)
  (= (modulo n 2) 1))

;exists-odd: albero -> true se almeno uno dei numeri contenuti
;sono dispari, false altrimenti
(define (exists-odd t)
  (cond
    [(odd? (tree-value t)) true]
    [else (exists-odd-children (tree-children t))]))
(define (exists-odd-children l)
  (cond
    [(empty? l) false]
    [(exists-odd (first l)) true]
    [else (exists-odd-children (rest l))]))

;esercizio: funzione pp (pretty printing)
;pp: albero -> una rappresentazione in forma di stringa dello
;stesso
;esempio: "1(2(7())5()4(5()6()))"
