(define-struct gentree (value children))
(define t (make-gentree 1
                        (list (make-gentree 2
                                            (list (make-gentree 7 empty)))
                              (make-gentree 5 empty)
                              (make-gentree 4
                                            (list (make-gentree 5 empty)
                                                  (make-gentree 6 empty))))))

;postvisit: albero -> lista di numeri
(define (postvisit t)
  (append (postvisit-children (gentree-children t))
          (list (gentree-value t))))

;postvisit-children: lista di alberi -> lista di numeri
(define (postvisit-children l)
  (cond
    [(empty? l) empty]
    [else
     (append (postvisit (first l))
             (postvisit-children (rest l)))]))

(define (pp t)
  (local ((define (pp-children l)
            (cond
              [(empty? l) ")"]
              [else (string-append (pp (first l))
                                   (pp-children (rest l)))])))
    (string-append (number->string (gentree-value t))
                   "("
                   (pp-children (gentree-children t)))))

;esercizio su alberi binari: BFS (breadth first visit)
;bfs: albero binario -> lista di numeri nell'ordine BFS
;ovvero: prima visita tutti i nodi a livello 1 (solo la radice)
;poi visita tutti i nodi a livello 2 (tutti i figli della
;radice), poi tutti i nodi a livello 3 e cosi' via
;consiglio: vi serve una coda

;elezioni
;testo disponibile qui:
;http://www.ccs.neu.edu/home/matthias/HtDP/Extended/elections.html
;1) dato un numero di seggi voglio generare i risultati delle
;   elezioni in ognuno di quei seggi
;2) dati i risultati delle elezioni in una lista di seggi
;   voglio calcolare il vincitore secondo le 3 strategie

;rappresentazione dei risultati nei vari seggi: usiamo una
;lista di liste; ogni lista contenuta ha lunghezza 3 e
;rappresenta le preferenze in quel seggio
(define risultati1
  (list
   (list 1 2 3)
   (list 2 3 1)
   (list 3 2 1)))

;do-election: number -> lista
;(la lista ha lunghezza pari al numero fornito in input ed ogni
;elemento e' una lista di lunghezza 3)
(define (do-election n)
  (cond
    [(= n 0) empty]
    [(< n 0) (error 'do-election "numero negativo")]
    [else (cons (preferenze empty)
                (do-election (- n 1)))]))

;isin: numero + lista di numeri -> true se il numero compare
;nella lista, false altrimenti
(define (isin v l)
  (cond
    [(empty? l) false]
    [(= v (first l)) true]
    [else (isin v (rest l))]))

;preferenze: argomento inutile -> lista di preferenze tra
;'pizza, 'tigelle, 'cinese
(define (preferenze arg)
  (local ((define (converti n)
            (cond
              [(= n 0) 'pizza]
              [(= n 1) 'cinese]
              [(= n 2) 'tigelle]))
          (define (vota esclusi) ;vota: lista di esclusi -> num
            (local ((define voto (random 3)))
              (cond [(isin voto esclusi) (vota esclusi)]
                    [else voto])))
          (define voto1 (random 3))
          (define voto2 (vota (list voto1)))
          (define voto3 (vota (list voto1 voto2))))
          (list (converti voto1) (converti voto2)
                (converti voto3))))

(define elezioni (do-election 20))

;rappresentazione dei punteggi ottenuti da ogni candidato ad
;un certo punto dello scrutinio: uso una lista associativa che
;associa ad ogni candidato i punti che ha ottenuto finora
(define-struct punteggio (nome punti))

;bump: lista di punteggi + nome + punti -> lista di punteggi
;data una lista di punteggi restituisce una nuova lista di
;punteggi nella quale il candidato fornito in input ha un
;numero di punti aumentato del valore fornito in input
(define (bump punteggi nome punti)
  (cond
    [(empty? punteggi) (list (make-punteggio nome punti))]
    [else
     (local ((define primo (first punteggi))
             (define nome-corrente (punteggio-nome primo)))
       (cond
         [(symbol=? nome-corrente nome)
          (cons (make-punteggio nome-corrente
                                (+ punti
                                   (punteggio-punti primo)))
                (rest punteggi))]
         [else (cons primo
                     (bump (rest punteggi) nome punti))]))]))

