;esercizio BFS

;definiamo il tipo di dato astratto "coda di numeri"
;costruttori
;- emptyqueue (coda vuota)
;- enqueue (aggiunge un numero ad una coda)

;emptyqueue: coda di numeri
(define emptyqueue empty)

;predicati
;- emptyqueue? (verifica se una coda vuota)
(define (emptyqueue? queue) (empty? queue))

;enqueue: numero + coda di numeri -> coda di numeri
;aggiunge un numero ad una coda di numeri
(define (enqueue n queue) (cons n queue))

;selettori
;- top (restituisce il primo numero arrivato nella coda)
;- dequeue (toglie il primo numero arrivato nella coda)

;top: coda di numeri -> numero
(define (top queue)
  (cond
    [(emptyqueue? queue) (error 'top "coda vuota")]
    [else
     (local ((define remaining (rest queue)))
       (cond
         [(emptyqueue? remaining) (first queue)]
         [else (top remaining)]))]))

;dequeue: coda di numeri -> coda di numeri
(define (dequeue queue)
  (cond
    [(emptyqueue? queue) (error 'dequeue "coda vuota")]
    [(emptyqueue? (rest queue)) empty]
    [else (cons (first queue) (dequeue (rest queue)))]))

(define q (enqueue 1 (enqueue 2 (enqueue 3 emptyqueue))))

;un albero binario di numeri e'
;- una tripla <valore, sx, dx> dove valore e' un numero e
;  sx, dx sono alberi binari di numeri
;- empty
(define-struct bintree (value sx dx))
(define t (make-bintree 1
                        (make-bintree 2
                                      (make-bintree 4 empty empty)
                                      (make-bintree 5 empty empty))
                        (make-bintree 3
                                      (make-bintree 6 empty empty)
                                      (make-bintree 7 empty empty))))

;bfs: albero binario di numeri -> lista di numeri
(define (bfs t)
  (local
;aux: coda di alberi binari di numeri -> lista di numeri
   ((define (aux queue)
    (cond
     [(emptyqueue? queue) empty]
     [else
      (local ((define next (top queue)))
       (cond
         [(empty? next) (aux (dequeue queue))]
         [else
          (cons (bintree-value next)
                (aux (enqueue (bintree-dx next)
                        (enqueue (bintree-sx next)
                                 (dequeue queue)))))]))])))
    (aux (enqueue t emptyqueue))))

;------------------------------------------------------------------------

;elezioni
;testo disponibile qui:
;http://www.ccs.neu.edu/home/matthias/HtDP/Extended/elections.html
;1) dato un numero di seggi voglio generare i risultati delle
;   elezioni in ognuno di quei seggi
;2) dati i risultati delle elezioni in una lista di seggi
;   voglio calcolare il vincitore secondo le 3 strategie

;rappresentazione dei risultati nei vari seggi: usiamo una
;lista di liste; ogni lista contenuta ha lunghezza 3 e
;rappresenta le preferenze in quel seggio
(define risultati1
  (list
   (list 1 2 3)
   (list 2 3 1)
   (list 3 2 1)))

;do-election: number -> lista
;(la lista ha lunghezza pari al numero fornito in input ed ogni
;elemento e' una lista di lunghezza 3)
(define (do-election n)
  (cond
    [(= n 0) empty]
    [(< n 0) (error 'do-election "numero negativo")]
    [else (cons (preferenze empty)
                (do-election (- n 1)))]))

;isin: numero + lista di numeri -> true se il numero compare
;nella lista, false altrimenti
(define (isin v l)
  (cond
    [(empty? l) false]
    [(= v (first l)) true]
    [else (isin v (rest l))]))

;preferenze: argomento inutile -> lista di preferenze tra
;'pizza, 'tigelle, 'cinese
(define (preferenze arg)
  (local ((define (converti n)
            (cond
              [(= n 0) 'pizza]
              [(= n 1) 'cinese]
              [(= n 2) 'tigelle]))
          (define (vota esclusi) ;vota: lista di esclusi -> num
            (local ((define voto (random 3)))
              (cond [(isin voto esclusi) (vota esclusi)]
                    [else voto])))
          (define voto1 (random 3))
          (define voto2 (vota (list voto1)))
          (define voto3 (vota (list voto1 voto2))))
    (list (converti voto1) (converti voto2)
          (converti voto3))))

(define elezioni (do-election 50))

;rappresentazione dei punteggi ottenuti da ogni candidato ad
;un certo punto dello scrutinio: uso una lista associativa che
;associa ad ogni candidato i punti che ha ottenuto finora
(define-struct punteggio (nome punti))

;bump: lista di punteggi + nome + punti -> lista di punteggi
;data una lista di punteggi restituisce una nuova lista di
;punteggi nella quale il candidato fornito in input ha un
;numero di punti aumentato del valore fornito in input
(define (bump punteggi nome punti)
  (cond
    [(empty? punteggi) (list (make-punteggio nome punti))]
    [else
     (local ((define primo (first punteggi))
             (define nome-corrente (punteggio-nome primo)))
       (cond
         [(symbol=? nome-corrente nome)
          (cons (make-punteggio nome-corrente
                                (+ punti
                                   (punteggio-punti primo)))
                (rest punteggi))]
         [else (cons primo
                     (bump (rest punteggi) nome punti))]))]))

;valuta1: risultati dei sottoseggi (lista di liste) -> lista di
;punteggi
(define (valuta1 elezioni)
  (cond
    [(empty? elezioni) empty]
    [else
     (local ((define risultato (first elezioni))
             (define punteggi (valuta1 (rest elezioni))))
       (bump punteggi (first risultato) 1))]))

(define (valuta2 elezioni)
  (cond
    [(empty? elezioni) empty]
    [else
     (local ((define risultato (first elezioni))
             (define punteggi (valuta2 (rest elezioni))))
       (bump
        (bump punteggi (first risultato) 1)
        (second risultato) 1))]))

(define (valuta3 elezioni)
  (cond
    [(empty? elezioni) empty]
    [else
     (local ((define risultato (first elezioni))
             (define punteggi (valuta3 (rest elezioni))))
       (bump
         (bump
          (bump punteggi (first risultato) 3)
          (second risultato) 2)
         (third risultato) 1))]))

;strategia elettorale: funzione che prende un risultato di
;sottoseggio, una lista di punteggi e restituisce una nuova
;lista di punteggi
;strategia: risultato + punteggi -> punteggi
(define (strategia1 risultato punteggi)
  (bump punteggi (first risultato) 1))
(define (strategia2 risultato punteggi)
  (bump
   (bump punteggi (first risultato) 1)
   (second risultato)
   1))
(define (strategia3 risultato punteggi)
  (bump
   (bump
    (bump punteggi (first risultato) 3)
    (second risultato)
    2)
   (third risultato)
   1))
;valuta: risultati dei sottoseggi + strategia -> lista di
;punteggi
;(NB valuta e' una funzione di ordine superiore che prende in
;input una funzione strategia)
(define (valuta elezioni strategia)
  (cond
    [(empty? elezioni) empty]
    [else
     (local ((define risultato (first elezioni))
             (define punteggi
               (valuta (rest elezioni) strategia)))
       (strategia risultato punteggi))]))

(define (ordinacrescente l) (quicksort l <=))
(define (ordinadecrescente l) (quicksort l >=))
(define (ordinavocabolario l) (quicksort l string<=?))
(define (ordinaacaso l)
  (local ((define (booleanoacaso x y) (= (random 2) 1)))
    (quicksort l booleanoacaso)))

;vincitore: elezioni + strategia -> il nome del vincitore
(define (vincitore elezioni strategia)
  (local ((define punteggi (valuta elezioni strategia))
          (define (punteggio>=? p1 p2)
            (>= (punteggio-punti p1) (punteggio-punti p2))))
    (punteggio-nome
     (first (quicksort punteggi punteggio>=?)))))

;esercizio: definite una funzione sort di ordine superiore
;sort: (lista di numeri) + (predicato di ordinamento) ->
;(lista di numeri)
;la lista restituita e' ordinato secondo il predicato di
;ordinamento fornito in input
;un predicato di ordinamento e' una funzione che prende due
;numeri e restituisce un booleano (true nel caso il primo
;numero debba comparira prima del secondo nell'ordinamento
;desiderato)
;si deve comportare come quicksort
