;tipo di dato astratto coda
;costruttori:
;- emptyqueue (coda vuota)
(define emptyqueue empty)
;- enqueue (aggiunge un elemento ad una coda)
;enqueue: elemento + coda -> coda
(define (enqueue v queue) (cons v queue))
;predicati:
;- emptyqueue? (mi dice se una coda e' vuota o meno)
;emptyqueue?: coda -> boolean
(define (emptyqueue? queue) (empty? queue))
;selettori:
;- top (restituisce il primo elemento della coda in ord. FIFO)
;top: coda -> elemento
(define (top queue)
  (cond
    [(emptyqueue? queue) (error 'top "coda vuota")]
    [else
     (local ((define restante (rest queue)))
       (cond
         [(emptyqueue? restante) (first queue)]
         [else (top restante)]))]))
;- dequeue (toglie il primo elemento della coda in ord. FIFO)
;dequeue: coda -> coda
(define (dequeue queue)
  (cond
    [(emptyqueue? queue) (error 'top "coda vuota")]
    [else
     (local ((define restante (rest queue)))
       (cond
         [(emptyqueue? restante) empty]
         [else (enqueue (first queue) (dequeue restante))]))]))

;albero binario di numeri
(define-struct bintree (value sx dx))
(define t (make-bintree 1
               (make-bintree 2
                    (make-bintree 4 empty empty)
                    (make-bintree 5 empty empty))
               (make-bintree 3
                    (make-bintree 6 empty empty)
                    (make-bintree 7 empty empty))))

;bfs: albero binario di numeri -> lista di numeri
(define (bfs t)
;aux: coda di alberi binari di numeri -> lista di numeri
  (local
   ((define (aux queue)
    (cond
      [(emptyqueue? queue) empty]
      [else
       (local ((define next (top queue))
             (define (enqueuene tree queue)
               (cond [(empty? tree) queue]
                     [else (enqueue tree queue)]))
             (define newqueue
              (enqueuene (bintree-dx next)
               (enqueuene (bintree-sx next) (dequeue queue)))))
       (cons (bintree-value next) (aux newqueue)))])))
    (aux (enqueue t emptyqueue))))

;------------------------------------------------------------------------

;elezioni
;rappresentazione dei risultati dei seggi
;utilizziamo una lista di liste, ogni elemento della lista
;rappresenta le preferenze ordinate (dalla piu' importante alla
;meno importante) uscenti da un dato seggio
;ogni preferenza e' un simbolo corrispondente al nome del
;candidato
;es con 2 seggi:
;(list (list 'tigelle 'sushi 'pizza)
;      (list 'sushi 'pizza 'tigelle))

;isin: numero + lista di numeri -> boolean
(define (isin n l)
  (cond
    [(empty? l) false]
    [(= (first l) n) true]
    [else (isin n (rest l))]))
;do-election: number -> risultati
;dato un numero di seggi genera i risultati delle elezioni in
;tali seggi; ovvero una lista di liste lunga n
(define (do-election n)
  (local ((define (vota esclusi) ;lista di numeri -> numero
            (local ((define voto (random 3)))
              (cond
                [(isin voto esclusi) (vota esclusi)]
                [else voto])))
          (define (converti voto) ;numero -> simbolo
            (cond
              [(= voto 0) 'tigelle]
              [(= voto 1) 'pizza]
              [(= voto 2) 'sushi]))
          (define (preferenze arg) ;arg inutile -> seggio
            (local ((define voto1 (vota empty))
                    (define voto2 (vota (list voto1)))
                    (define voto3 (vota (list voto1 voto2))))
              (list (converti voto1) (converti voto2)
                    (converti voto3)))))          
  (cond
    [(= n 0) empty]
    [(< n 0) (error 'do-election "n. seggi negativo")]
    [else (cons (preferenze empty)
                (do-election (- n 1)))])))

(define elezioni (do-election 50))

;punteggio e' una coppia che associa ad un candidato il numero
;di punti che ha ricevuto fino ad ora
(define-struct punteggio (nome punti))

;bump: lista di punteggi + candidato + numero -> lista di punt.
;aumenta il numero di punti del candidato di una certa
;quantita' e restituisce la nuova lista di punteggi
(define (bump punteggi nome punti)
  (cond
    [(empty? punteggi) (list (make-punteggio nome punti))]
    [else
     (local ((define punteggio (first punteggi))
             (define nomecorr (punteggio-nome punteggio)))
      (cond
        [(symbol=? nomecorr nome)
           (cons
              (make-punteggio nomecorr
                (+ punti (punteggio-punti punteggio)))
              (rest punteggi))]
        [else (cons punteggio
                    (bump (rest punteggi) nome punti))]))]))

