;elezioni
;rappresentazione dei risultati dei seggi
;utilizziamo una lista di liste, ogni elemento della lista
;rappresenta le preferenze ordinate (dalla piu' importante alla
;meno importante) uscenti da un dato seggio
;ogni preferenza e' un simbolo corrispondente al nome del
;candidato
;es con 2 seggi:
;(list (list 'tigelle 'sushi 'pizza)
;      (list 'sushi 'pizza 'tigelle))

;isin: numero + lista di numeri -> boolean
(define (isin n l)
  (cond
    [(empty? l) false]
    [(= (first l) n) true]
    [else (isin n (rest l))]))
;do-election: number -> risultati
;dato un numero di seggi genera i risultati delle elezioni in
;tali seggi; ovvero una lista di liste lunga n
(define (do-election n)
  (local ((define (vota esclusi) ;lista di numeri -> numero
            (local ((define voto (random 3)))
              (cond
                [(isin voto esclusi) (vota esclusi)]
                [else voto])))
          (define (converti voto) ;numero -> simbolo
            (cond
              [(= voto 0) 'tigelle]
              [(= voto 1) 'pizza]
              [(= voto 2) 'sushi]))
          (define (preferenze arg) ;arg inutile -> seggio
            (local ((define voto1 (vota empty))
                    (define voto2 (vota (list voto1)))
                    (define voto3 (vota (list voto1 voto2))))
              (list (converti voto1) (converti voto2)
                    (converti voto3)))))          
    (cond
      [(= n 0) empty]
      [(< n 0) (error 'do-election "n. seggi negativo")]
      [else (cons (preferenze empty)
                  (do-election (- n 1)))])))

(define elezioni (do-election 50))

;punteggio e' una coppia che associa ad un candidato il numero
;di punti che ha ricevuto fino ad ora
(define-struct punteggio (nome punti))

;bump: lista di punteggi + candidato + numero -> lista di punt.
;aumenta il numero di punti del candidato di una certa
;quantita' e restituisce la nuova lista di punteggi
(define (bump punteggi nome punti)
  (cond
    [(empty? punteggi) (list (make-punteggio nome punti))]
    [else
     (local ((define punteggio (first punteggi))
             (define nomecorr (punteggio-nome punteggio)))
       (cond
         [(symbol=? nomecorr nome)
          (cons
           (make-punteggio nomecorr
                           (+ punti (punteggio-punti punteggio)))
           (rest punteggi))]
         [else (cons punteggio
                     (bump (rest punteggi) nome punti))]))]))


;punteggi1: risultati delle elezioni (lista di liste ...) ->
;           lista di punteggi
;i punteggi sono calcolati secondo la strategia 1 (1 punto al
;primo classificato in ogni seggio)
(define (punteggi1 elezioni)
  (cond
    [(empty? elezioni) empty]
    [else
     (local ((define seggio (first elezioni))
             (define risultati (punteggi1 (rest elezioni))))
       (bump risultati (first seggio) 1))]))

;come punteggi1, ma implementa la seconda strategia (i.e. 1
;punto al primo classificato di ogni seggio e 1 punto al
;secondo)
(define (punteggi2 elezioni)
  (cond
    [(empty? elezioni) empty]
    [else
     (local ((define seggio (first elezioni))
             (define risultati (punteggi2 (rest elezioni))))
       (bump
        (bump risultati (first seggio) 1)
        (second seggio)
        1))]))

;come punteggi1 ma implementa la terza strategia (3 punti al
;primo classificato, 2 al secondo e 1 al terzo)
(define (punteggi3 elezioni)
  (cond
    [(empty? elezioni) empty]
    [else
     (local ((define seggio (first elezioni))
             (define risultati (punteggi3 (rest elezioni))))
       (bump
        (bump
         (bump risultati (first seggio) 3)
         (second seggio)
         2)
        (third seggio)
        1))]))

;strategia: e' una funzione che ha questo prototipo
; lista di punteggi + risultati di un seggio -> lista di punt.

(define (strategia1 punteggi seggio)
  (bump punteggi (first seggio) 1))
(define (strategia2 punteggi seggio)
  (bump
   (bump punteggi (first seggio) 1)
   (second seggio)
   1))
(define (strategia3 punteggi seggio)
  (bump
   (bump
    (bump punteggi (first seggio) 3)
    (second seggio)
    2)
   (third seggio)
   1))

;punteggi: lista di risultati (lista di liste ...) +
;          strategia -> una lista di punteggi
;i punteggi sono calcolati in base alla strategia fornita in
;input
;N.B. punteggi e' una funzione di ordine superiore perche'
;prende in input la _funzione_ strategia
(define (punteggi elezioni strategia)
  (cond
    [(empty? elezioni) empty]
    [else
     (local ((define seggio (first elezioni))
             (define risultati
               (punteggi (rest elezioni) strategia)))
       (strategia risultati seggio))]))

;ordinamento di numeri crescente
(define (ordinacrescenti l) (quicksort l <=))
(define (ordinadecrescenti l) (quicksort l >=))
(define (ordinaacaso l)
  (local ((define (randombool x y) (= (random 2) 0)))
    (quicksort l randombool)))
(define (ordinaalfabeto l) (quicksort l string<=?))

;esercizio: definite una funzione sort (analoga a quella vista
;a lezione) che abbia lo stesso prototipo di quicksort
;sort: lista di elementi + predicato di ordinamento -> lista di
;elementi
;il predicato di ordinamento e' una funzione: x + x -> boolean
;la lista restituita deve essere ordinata secondo il predicato
;di ordinamento
;vietato usare quicksort!

;ordinapunteggi: lista di punteggi -> lista di punteggi
;la lista restitutita e' ordinata in maniera decrescente sul
;numero di punti
(define (ordinapunteggi l)
  (local ((define (punteggio>=? p1 p2)
            (>= (punteggio-punti p1) (punteggio-punti p2))))
    (quicksort l punteggio>=?)))

;vincitore: risultati elettorali + strategia -> nome del ;vincitore
;N.B. assumiamo che non ci siano parimerito per semplicita'
;N.BB. l'assunzione di cui sopra e' falsa nel nostro codice
(define (vincitore elezioni strategia)
  (punteggio-nome
   (first (ordinapunteggi (punteggi elezioni strategia)))))
