;empty-win: arg inutile -> finestra
(define (empty-win dummy)
  (create-window empty))
;esempio: (empty-win empty)

;hello: string -> finestra contenente il messaggio fornito in
;input
(define (hello testo)
  (local ((define msg (make-message testo)))
    (create-window (list (list msg)))))
;esempio: (hello "Hello, world!")

;hello2: string -> finestra
;analoga alla funzione precedente, ma il messaggio viene
;mostrato all'utente solo quando il bottone viene clickato
(define (hello2 testo)
  (local ((define msg (make-message ""))
          (define (button-callback e)
            ; scrive la stringa testo nel messaggio msg
            (draw-message msg testo))
          (define button (make-button "go" button-callback)))
    (create-window (list (list msg button)))))
;esempio: (hello2 "Hello, world!")

;hello3: string -> finestra
(define (hello3 testo)
  (local ((define msg (make-message ""))
          (define (go-callback e)
            (draw-message msg testo))
          (define (clear-callback e)
            (draw-message msg ""))
          (define go-button (make-button "go" go-callback))
          (define clear-button
            (make-button "clear" clear-callback)))
    (create-window (list (list msg)
                         (list go-button clear-button)))))
;esempio: (hello3 "Hello, world!")

;copy: arg -> finestra
;la finestra permette all'utente di inserire testo e lo copia
;in un messaggio quando l'utente preme il tasto "copy"
(define (copy dummy)
  (local ((define msg (make-message ""))
          (define text (make-text "input:"))
          (define (copy-callback e)
            (draw-message msg (text-contents text)))
          (define copy-button
            (make-button "copy" copy-callback)))
    (create-window (list (list text copy-button)
                         (list msg)))))
;esempio: (copy empty)

;vote: lista di stringhe -> finestra
(define (vote choices)
  (local ((define msg (make-message ""))
          (define choice (make-choice choices))
          (define (vote-callback e)
            (draw-message msg
                          (list-ref choices (choice-index choice))))
          (define vote-button
            (make-button "vote" vote-callback)))
    (create-window (list (list choice vote-button)
                         (list msg)))))
;esempio: (vote (list "pizza" "sushi" "tigelle"))

;un contatto e' una coppia <nome, indirizzo email>
;sia nome che indirizzo sono stringhe
(define-struct contact (name email))

;lista di contact di esempio
(define contactlist
  (list
   (make-contact "zack" "zack@cs.unibo.it")
   (make-contact "nadia" "busi@cs.unibo.it")))

;lookup: lista di contatti + stringa -> stringa o false
;data in inputa una lista di contatti ed un nome, restituisce
;l'indirizzo email associato al nome oppure false nel caso
;in cui il nome non sia presente nella lista
(define (lookup contactlist name)
  (cond
    [(empty? contactlist) false]
    [else
     (local ((define currcontact (first contactlist)))
       (cond
         [(string=? name (contact-name currcontact))
          (contact-email currcontact)]
         [else (lookup (rest contactlist) name)]))]))
;esempio: (lookup contactlist "zack")

;agenda: lista di contact -> finestra
(define (agenda contactlist)
  (local ((define msg (make-message ""))
          (define name (make-text "name:"))
          (define (find-callback e)
            (local
                ((define email
                   (lookup contactlist (text-contents name))))
              (cond
                [(string? email) (draw-message msg email)]
                [else (draw-message msg "not found")])))
          (define find-button
            (make-button "find" find-callback)))
    (create-window (list (list name find-button)
                       (list msg)))))
;esempio: (agenda contactlist)

