
;esercizio volta scorsa: ordinamento con predicato
;sort: predicato di ordinamento + lista -> lista (ordinata
;secondo il predicato di ordinamento)
(define (sort le l)
  (cond
    [(empty? l) empty]
    [else (insert le (first l) (sort le (rest l)))]))

;insert: predicato di ordinamento + elemento +
;lista -> lista (ordinata)
;inserisce un elemento in una lista ordinata,
;rispettando l'ordine imposto dal predicato di ordinamento
(define (insert le n l)
  (cond
    [(empty? l) (list n)]
    [else
     (local ((define primo (first l)))
       (cond
         [(le n primo) (cons n l)]
         [else (cons primo (insert le n (rest l)))]))]))

(define even-10 (build-list 10 (lambda (x) (* 2 x))))
(define odd-10 (build-list 10 (lambda (x) (+ 1 (* 2 x)))))
(define sum-even-10 (foldl + 0 even-10))
(define sum-odd-10 (foldl + 0 odd-10))

;sum-even: naturale -> naturale
;dato in input la lunghezza della sequenza di numeri pari,
;mi restituisce la sua somma
;(sum-even 0) -> 0
;(sum-even 1) -> 2
;(sum-even 2) -> 6
;...
(define (sum-even n)
  (cond
    [(= n 0) 0]
    [else (+ (* n 2) (sum-even (- n 1)))]))

;rappresentazione scheme di sequenze (infinite)
;uso una funzione che dato l'i-esimo numero naturale mi
;restituisce l'i-esimo elemento della mia sequenza
;esempi:
(define (even-seq i) (* 2 i))
(define (odd-seq i) (+ 1 (* 2 i)))

;sum: (naturale -> naturale) + naturale -> naturale
;il valore di ritorno e' la somma dei primi n elementi della
;sequenza dove n e' il naturale fornito in input
(define (sum seq n)
  (cond
    [(= n 0) (seq 0)]
    [else (+ (seq n) (sum seq (- n 1)))]))

;mk-taylor-seq: numero -> (naturale -> numero)
;dato in input un numero x mi restituisce la sequenza di taylor
;della funzione e^x per quel dato x
(define (mk-taylor-seq x)
  (local ((define (fact n)
            (cond
              [(= n 0) 1]
              [else (* n (fact (- n 1)))]))
          (define (power n k)
            (cond
              [(= k 0) 1]
              [else (* n (power n (- k 1)))])))
  (lambda (i)
    (cond
      [(= i 0) 1]
      [else (/ (power x i) (fact i))]))))

;myexp: naturale -> numero
;calcola e^x
(define (myexp x)
  (sum (mk-taylor-seq x) 100))

;integrate: (numero->numero) + numero + numero -> numero
;calcola l'integrale della funzione tra i due punti dati in
;input
(define (integrate f a b)
  (local ((define x0 a)
          (define x1 (/ (+ a b) 2))
          (define x2 b)
          (define y0 (f x0))
          (define y1 (f x1))
          (define y2 (f x2))
          (define area1 (/ (* (+ y0 y1) (- x1 x0)) 2))
          (define area2 (/ (* (+ y1 y2) (- x2 x1)) 2)))
    (+ area1 area2)))

;esercizio: scrivere una funzione di integrazione "migliore" di
;integrate. Questa funzione deve prendere in input il numero di
;intervalli nei quali l'intervallo a-b deve essere diviso. Su
;ognuno di tali intervalli utilizza la funzione integrate.

;derivate: (numero -> numero) -> (numero -> numero)
;data in input una funzione, restituisce la sua derivata
(define (derivate f)
  (local ((define epsilon 0.1))
    (lambda (x)
      (/ (- (f (+ x epsilon)) (f (- x epsilon)))
         (* 2 epsilon)))))

;bisect: (numero->numero) + numero + numero + numero -> numero
;data in input una funzione monotona crescente, un intervallo
;[a,b] ed una tolleranza, restituisce lo zero della funzione
;ovvero un punto x t.c. (abs (f x)) <= tolleranza
(define (bisect f a b delta)
  (local ((define x0 a)
          (define x1 (/ (+ a b) 2))
          (define x2 b)
          (define y0 (f x0))
          (define y1 (f x1))
          (define y2 (f x2)))
    (cond
      [(<= (abs y1) delta) x1]
      [(>= y1 0) (bisect f x0 x1 delta)]
      [else (bisect f x1 x2 delta)])))